# -*- coding: utf-8 -*-
# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

""" Default Karaage Settings. """
from socket import getfqdn

import django
import environ
import six


HTTP_HOST = getfqdn()
env = environ.Env()

###
# DJANGO SETTINGS
###

# A boolean that turns on/off debug mode.
#
# Never deploy a site into production with DEBUG turned on.
#
# Did you catch that? NEVER deploy a site into production with DEBUG turned on.
#
# One of the main features of debug mode is the display of detailed error
# pages.  If your app raises an exception when DEBUG is True, Django will
# display a detailed traceback, including a lot of metadata about your
# environment, such as all the currently defined Django settings (from
# settings.py).
DEBUG = False

# Do not change: The model to use to represent a Django user.
#
# If false, administrators can log in but don't get any special access. Used
# for karaage-registration.
AUTH_USER_MODEL = 'karaage.Person'

# Do not change: A tuple of strings designating all applications that are
# enabled in this Django installation. Each string should be a dotted Python
# path to:
#
# * an application configuration class, or a package containing a
# * application.
if django.VERSION < (1, 7):
    KARAAGE_APPS = (
        'karaage',
    )
else:
    KARAAGE_APPS = (
        'karaage.apps.Karaage',
    )

INSTALLED_APPS = (
    'django_xmlrpc',
    'captcha',
    'ajax_select',
    'jsonfield',
    'django_tables2',
    'tldap.django',
    'pipeline',
    'django_celery_results',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.humanize',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.auth',
)

# A list containing the settings for all template engines to be used with
# Django. Each item of the list is a dictionary containing the options for an
# individual engine.
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            "/etc/karaage3/templates",
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'karaage.common.context_processors.common',
                'django.contrib.auth.context_processors.auth',
                'django.template.context_processors.request',
                'django.template.context_processors.debug',
                'django.template.context_processors.i18n',
                'django.template.context_processors.media',
                'django.template.context_processors.static',
                'django.template.context_processors.tz',
                'django.contrib.messages.context_processors.messages',
            ],
            'debug': True,
            # 'string_if_invalid': InvalidString("%s"),
        },
    },
]

# A boolean that specifies whether Django’s translation system should be
# enabled. This provides an easy way to turn it off, for performance. If this
# is set to False, Django will make some optimizations so as not to load the
# translation machinery.
USE_I18N = False

# A tuple of middleware classes to use.
MIDDLEWARE = [
    'django.middleware.common.CommonMiddleware',
    'django.middleware.common.BrokenLinkEmailsMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'karaage.middleware.threadlocals.ThreadLocals',
    'karaage.middleware.saml.SamlUserMiddleware',
    'tldap.django.middleware.TransactionMiddleware',
    'whitenoise.middleware.WhiteNoiseMiddleware',
]

# A tuple of authentication backend classes (as strings) to use when attempting
# to authenticate a user.

# The ``karaage.backends.LDAPBackend`` backend is legacy, to support upgrades
# from Karaage versions before 3.
AUTHENTICATION_BACKENDS = (
    'django.contrib.auth.backends.ModelBackend',
    'karaage.backends.LDAPBackend',
)

PASSWORD_HASHERS = [
    'django.contrib.auth.hashers.BCryptSHA256PasswordHasher',
    'django.contrib.auth.hashers.BCryptPasswordHasher',
    'django.contrib.auth.hashers.PBKDF2PasswordHasher',
    'django.contrib.auth.hashers.PBKDF2SHA1PasswordHasher',
    # Commented out as not supported by Django 1.8:
    # 'django.contrib.auth.hashers.Argon2PasswordHasher',
    'django.contrib.auth.hashers.SHA1PasswordHasher',
]

# The default value for the X-Frame-Options header used by
# XFrameOptionsMiddleware. See the `clickjacking protection
# <https://docs.djangoproject.com/en/dev/ref/clickjacking/>`_ documentation.
X_FRAME_OPTIONS = 'DENY'

# Whether to use a secure cookie for the session cookie. If this is set to
# True, the cookie will be marked as “secure,” which means browsers may ensure
# that the cookie is only sent under an HTTPS connection.
SESSION_COOKIE_SECURE = True

# Whether to expire the session when the user closes their browser. See
# `Browser-length sessions vs. persistent sessions
# <https://docs.djangoproject.com/en/dev/topics/http/sessions/
# browser-length-vs-persistent-sessions>`_.
SESSION_EXPIRE_AT_BROWSER_CLOSE = True

# A string representing the full Python import path to your root URLconf. For
# example: "mydjangoapps.urls". Can be overridden on a per-request basis by
# setting the attribute urlconf on the incoming HttpRequest object.
ROOT_URLCONF = 'karaage.conf.urls'

# The URL where requests are redirected for login, especially when
# using the login_required() decorator. Mostly not used anymore.
LOGIN_URL = 'kg_profile_login'

# The absolute path to the directory where collectstatic will collect static
# files for deployment.
STATIC_ROOT = '/var/lib/karaage3/static'


# URL to use when referring to static files located in STATIC_ROOT.
STATIC_URL = '/kgstatic/'

# A list of strings representing the host/domain names that this Django site
# can serve. This is a security measure to prevent an attacker from poisoning
# caches and password reset emails with links to malicious hosts by submitting
# requests with a fake HTTP Host header, which is possible even under many
# seemingly-safe web server configurations.
ALLOWED_HOSTS = ["%(HOST)s"]

###
# DJANGO PIPELINE
###

STATICFILES_STORAGE = 'pipeline.storage.PipelineCachedStorage'

PIPELINE = {
    'EMBED_PATH': r'img/|images/',
    'CSS_COMPRESSOR': 'pipeline.compressors.cssmin.CSSMinCompressor',
    'STYLESHEETS': {
        'karaage': {
            'source_filenames': (
                'css/*.css',
                'ajax_select/css/ajax_select.css',
            ),
            'output_filename': 'min.css',
            'variant': 'datauri',
        },
    },
    'JS_COMPRESSOR': 'pipeline.compressors.slimit.SlimItCompressor',
    'JAVASCRIPT': {
        'karaage': {
            'source_filenames': (
                'js/jquery-1.11.2.js',
                'js/jquery-ui-1.11.4.js',
                'js/*.js',
                'ajax_select/js/ajax_select.js',
            ),
            'output_filename': 'min.js',
        }
    },
}


###
# AJAX SETTINGS
###

# Channels for django-ajax-selects.
AJAX_LOOKUP_CHANNELS = {
    'person': ('karaage.people.lookups', 'PersonLookup'),
    'group': ('karaage.people.lookups', 'GroupLookup'),
    'institute': ('karaage.institutes.lookups', 'InstituteLookup'),
    'project': ('karaage.projects.lookups', 'ProjectLookup'),
}

# Automatically load media files required for django-ajax-selects.
AJAX_SELECT_BOOTSTRAP = True


##
# XMLRPC
##

# List of all XMLRPC methods that we support.
XMLRPC_METHODS = (
    ('karaage.machines.xmlrpc.get_disk_quota', 'get_disk_quota',),
    ('karaage.projects.xmlrpc.get_project', 'get_project',),
    ('karaage.projects.xmlrpc.get_project_members', 'get_project_members',),
    ('karaage.projects.xmlrpc.get_projects', 'get_projects',),
    ('karaage.projects.xmlrpc.get_users_projects', 'get_users_projects',),
)


###
# KARAAGE SETTINGS
###

# SETTINGS FROM DOCKER
BUILD_DATE = env('BUILD_DATE', default=None)
VCS_REF = env('VCS_REF', default=None)
SLURM_VER = env('SLURM_VER', default=None)

# DIRECTORY FOR TEMP FILES
TMP_DIR = "/var/cache/karaage3/tmp/"

# DIRECTORY FOR SHARED FILES
FILES_DIR = "/var/lib/karaage3/files/"

# URL FOR SHARED FILES
FILES_URL = "/kgfiles/"

# Default machine category, used by legacy XMLRPC when client doesn't specify
# a machine name.
DEFAULT_MC = 1

# List of shells that we support.
SHELLS = (
    ('/bin/bash', 'bash'),
    ('/bin/csh', 'csh'),
    ('/bin/ksh', 'ksh'),
    ('/bin/tcsh', 'tcsh'),
    ('/bin/zsh', 'zsh'),
)

# Default shell we should use for new accounts.
DEFAULT_SHELL = '/bin/bash'

# List of global datastores and configurations for Karaage.
DATASTORES = [
]

# Can any logged in user access the usage information?
USAGE_IS_PUBLIC = True

# Do we support shibboleth for logins?
SHIB_SUPPORTED = False

# How do we interpret HTTP parameters as shibboleth values?
SHIB_ATTRIBUTE_MAP = {
    "HTTP_SHIB_IDENTITY_PROVIDER": (True, "idp"),
    "HTTP_PERSISTENT_ID": (True, "persistent_id"),
    "HTTP_MAIL": (True, "email"),
    "HTTP_GIVENNAME": (False, "first_name"),
    "HTTP_SN": (False, "last_name"),
    "HTTP_TELEPHONENUMBER": (False, "telephone"),
    "HTTP_UID": (False, "uid"),
}

# RE for validing a username.
USERNAME_VALIDATION_RE = r'[-\w]+'

# Error to display if username validation failed.
USERNAME_VALIDATION_ERROR_MSG = six.u(
    'Usernames can only contain letters, '
    'numbers and underscores')
USERNAME_MAX_LENGTH = 255

# RE for validing a project.
PROJECT_VALIDATION_RE = r'[-\w]+'

# Max length of a project identifier
PROJECT_ID_MAX_LENGTH = 255

# Error to display if project validation failed.
PROJECT_VALIDATION_ERROR_MSG = six.u(
    'Project names can only contain letters, '
    'numbers and underscores')

# RE for validing a group id.
GROUP_VALIDATION_RE = r'[-\w]+'

# Error to display if group validation failed.
GROUP_VALIDATION_ERROR_MSG = six.u(
    'Group names can only contain letters, '
    'numbers and underscores')

# Used for lock_training_accounts and unlock_training_accounts commands.
TRAINING_ACCOUNT_PREFIX = '^train[0-9]+$'

# Registration base URL - Used in email templates.
REGISTRATION_BASE_URL = 'https://%(HOST)s/karaage'

# Admin base URL - Used in email templates.
ADMIN_BASE_URL = 'https://%(HOST)s/karaage'

# If true, only administrators can log on to site. Used for karaage-admin.
ADMIN_REQUIRED = False

# If false, administrators can log in but don't get any special access. Used
# for karaage-registration.
ADMIN_IGNORED = False

# Available formatting that can be used for displaying datetime fields on
# templates.
SHORT_DATE_FORMAT = "Y-m-d"
SHORT_DATETIME_FORMAT = "Y-m-d H:i"

# List of Karaage plugins
PLUGINS = []

# The name of the class to use for starting the test suite.
TEST_RUNNER = 'django.test.runner.DiscoverRunner'

# A list of identifiers of messages generated by the system check framework
# (i.e. ["models.W001"]) that you wish to permanently acknowledge and ignore.
# Silenced warnings will no longer be output to the console; silenced errors
# will still be printed, but will not prevent management commands from running.
SILENCED_SYSTEM_CHECKS = [
    # BooleanField does not have a default value
    '1_6.W002',
]

CELERY_ENABLE_UTC = True
CELERY_BROKER_URL = 'redis://localhost:6379/0'
CELERY_RESULT_BACKEND = 'django-db'
CELERY_BEAT_SCHEDULE = {
    'daily': {
        'task': 'karaage.tasks.daily',
        'schedule': 6 * 60 * 60,
    },
}

APPLICATION_PROJECT = {
    'start': {
        'type': 'transition',
        'class': 'karaage.plugins.kgapplications.views.transitions.TransitionOpen',
        'on_success': {
            'type': 'goto',
            'key': 'O',
        },
    },
    'O': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.aed.StateApplicantEnteringDetails',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        },
        'on_duplicate': {
            'type': 'goto',
            'key': 'DUP',
        },
        'on_submit': {
            'type': 'transition',
            'class': 'karaage.plugins.kgapplications.views.transitions.TransitionSubmit',
            'on_success': {
                'type': 'transition',
                'class': 'karaage.plugins.kgapplications.views.transitions.TransitionSplit',
                'on_existing_project': {
                    'type': 'goto',
                    'key': 'L',
                },
                'on_new_project': {
                    'type': 'goto',
                    'key': 'D',
                },
                'on_error': {
                    'type': 'goto',
                    'key': 'R',
                },
            },
            'on_error': {
                'type': 'goto',
                'key': 'R',
            },
        }
    },
    'L': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateWaitingForLeader',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_duplicate': {
            'type': 'goto',
            'key': 'DUP',
        },
        'on_approve': {
            'type': 'goto',
            'key': 'K',
        },
    },
    'D': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateWaitingForDelegate',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_duplicate': {
            'type': 'goto',
            'key': 'DUP',
        },
        'on_approve': {
            'type': 'goto',
            'key': 'K',
        },
    },
    'K': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateWaitingForAdmin',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_duplicate': {
            'type': 'goto',
            'key': 'DUP',
        },
        'on_approve': {
            'type': 'transition',
            'class': 'karaage.plugins.kgapplications.views.transitions.TransitionApprove',
            'on_password_needed': {
                'type': 'goto',
                'key': 'P',
            },
            'on_password_ok': {
                'type': 'goto',
                'key': 'C',
            },
            'on_error': {
                'type': 'goto',
                'key': 'K',
            },
        },
    },
    'P': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StatePassword',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_submit': {
            'type': 'goto',
            'key': 'C',
        },
    },
    'C': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateCompleted',
        'on_archive': {
            'type': 'goto',
            'key': 'A',
        },
    },
    'A': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateArchived',
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        }
    },
    'R': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateDeclined',
        'on_archive': {
            'type': 'goto',
            'key': 'A',
        },
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        }
    },
    'DUP': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateDuplicateApplicant',
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        },
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
    },
}


APPLICATION_SOFTWARE = {
    'start': {
        'type': 'transition',
        'class': 'karaage.plugins.kgapplications.views.transitions.TransitionOpen',
        'on_success': {
            'type': 'goto',
            'key': 'O',
        },
    },
    'O': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateIntroduction',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_submit': {
            'type': 'transition',
            'class': 'karaage.plugins.kgapplications.views.transitions.TransitionSubmit',
            'on_success': {
                'type': 'goto',
                'key': 'K'
            },
            'on_error': {
                'type': 'goto',
                'key': 'R'
            },
        }
    },
    'K': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateWaitingForAdminSoftware',
        'on_cancel': {
            'type': 'goto',
            'key': 'R',
        },
        'on_duplicate': {
            'type': 'goto',
            'key': 'DUP',
        },
        'on_approve': {
            'type': 'transition',
            'class': 'karaage.plugins.kgapplications.views.transitions.TransitionApprove',
            'on_password_needed': {
                'type': 'goto',
                'key': 'R',
            },
            'on_password_ok': {
                'type': 'goto',
                'key': 'C',
            },
            'on_error': {
                'type': 'goto',
                'key': 'R',
            },
        },
    },
    'C': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateCompleted',
        'on_archive': {
            'type': 'goto',
            'key': 'A',
        },
    },
    'A': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateArchived',
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        }
    },
    'R': {
        'type': 'state',
        'class': 'karaage.plugins.kgapplications.views.states.StateDeclined',
        'on_archive': {
            'type': 'goto',
            'key': 'A',
        },
        'on_reopen': {
            'type': 'goto',
            'key': 'start',
        }
    },
}



#!/usr/bin/env python

# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.
"""Karaage setup script."""

import sys
import os

from setuptools import Command, setup, find_packages


VERSION='5.0.14'


class VerifyVersionCommand(Command):
    """Custom command to verify that the git tag matches our version"""
    description = 'verify that the git tag matches our version'
    user_options = [
      ('version=', None, 'expected version'),
    ]

    def initialize_options(self):
        self.version = None

    def finalize_options(self):
        pass

    def run(self):
        version = self.version

        if version != VERSION:
            info = "{0} does not match the version of this app: {1}".format(
                version, VERSION
            )
            sys.exit(info)


def fullsplit(path, result=None):
    """
    Split a pathname into components (the opposite of os.path.join) in a
    platform-neutral way.
    """
    if result is None:
        result = []
    head, tail = os.path.split(path)
    if head == '':
        return [tail] + result
    if head == path:
        return result
    return fullsplit(head, [tail] + result)


dirs = ['karaage', ]
packages = []
for d in dirs:
    for dirpath, dirnames, filenames in os.walk(d):
        # Ignore dirnames that start with '.'
        for i, dirname in enumerate(dirnames):
            if dirname.startswith('.'):
                del dirnames[i]
        if filenames:
            packages.append('.'.join(fullsplit(dirpath)))

tests_require = [
    "django-extensions",
    "factory_boy",
    "mock",
    "cracklib",
    "pytest",
    "pytest-runner",
]

setup(
    name="karaage",
    version=VERSION,
    url='https://github.com/Karaage-Cluster/karaage',
    author='Brian May',
    author_email='brian@v3.org.au',
    description='Collection of Django apps to manage a clusters',
    packages=packages,
    license="GPL3+",
    long_description=open('README.rst').read(),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: GNU General Public "
            "License v3 or later (GPLv3+)",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    keywords="karaage cluster user administration",
    package_data={
        '': ['*.css', '*.html', '*.js', '*.png', '*.gif', '*.map', '*.txt',
             '*.json'],
    },
    scripts=[
        'sbin/kg_set_secret_key',
        'sbin/kg-manage',
    ],
    install_requires=[
        "cssmin",
        "Django >= 1.8",
        "python-alogger >= 2.0",
        "django-xmlrpc >= 0.1",
        "django-simple-captcha",
        "django-ajax-selects >= 1.1.3",
        "django_jsonfield >= 0.9.12",
        "django-model-utils >= 2.0.0",
        "python-tldap >= 0.3.3",
        "django-pipeline >= 1.6.0",
        "django-tables2",
        "django-filter",
        "django-environ",
        "six",
        "slimit>=0.8.1",
    ],
    tests_require=tests_require,
    cmdclass={
        'verify': VerifyVersionCommand,
    },
    extras_require={
        'tests': tests_require,
        'applications': [
            # no dependencies for kgapplications
        ],
        'software': [
            "karaage[applications]",
            "karaage[usage]",
        ],
        'usage': [
            "karaage[software]",
            "django_celery",
            "matplotlib",
        ],
    },
)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import sys

import django.db.transaction
import tldap.transaction
from django.core.management.base import BaseCommand, CommandError

from karaage.people.models import Person
from karaage.people.utils import (
    UsernameInvalid,
    UsernameTaken,
    validate_username_for_rename_person,
)


try:
    input = raw_input
except NameError:
    pass


class Command(BaseCommand):
    help = 'Change a username for a person and all accounts for that person'
    args = '<old username> <new username>'

    @django.db.transaction.atomic
    @tldap.transaction.commit_on_success
    def handle(self, *args, **options):
        if len(args) != 2:
            raise CommandError(
                'Usage: change_username <old username> <new username>')

        old = args[0]
        new = args[1]

        try:
            person = Person.objects.get(username=old)
        except Person.DoesNotExist:
            raise CommandError('person %s does not exist' % old)

        try:
            validate_username_for_rename_person(new, person)
        except UsernameInvalid as e:
            raise CommandError(e.args[0])
        except UsernameTaken as e:
            raise CommandError(e.args[0])

        while 1:
            confirm = input(
                'Change person "%s" and accounts to "%s (yes,no): '
                % (old, new))
            if confirm == 'yes':
                break
            elif confirm == 'no':
                return sys.exit(0)
            else:
                print("Please enter yes or no")

        for account in person.account_set.filter(date_deleted__isnull=True):
            account.username = new
            account.save()
            print("Changed username to %s" % account.username)

        person.username = new
        person.save()
        print("Changed username on person")

        print("Done")



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django_xmlrpc.decorators import xmlrpc_func

from karaage.common.decorators import xmlrpc_machine_required

from .alogger import parse_logs
from .models import UsedModules


@xmlrpc_machine_required()
@xmlrpc_func(returns='string', args=['array', 'date', 'string', 'string'])
def parse_usage(machine, usage, date, machine_name, log_type):
    """
    Parses usage
    """
    assert machine.name == machine_name

    year, month, day = date.split('-')
    date = datetime.date(int(year), int(month), int(day))

    return parse_logs(usage, date, machine_name, log_type)


@xmlrpc_machine_required()
@xmlrpc_func(returns='string', args=['string', 'array', 'string'])
def add_modules_used(machine, jobid, modules, date):
    year, month, day = date.split('-')
    date = datetime.date(int(year), int(month), int(day))

    try:
        um = UsedModules.objects.get(jobid=jobid)
    except UsedModules.DoesNotExist:
        um = UsedModules()
        um.jobid = jobid
        um.date_added = date

    assert ":" not in modules
    um.modules = ":".join(modules)
    um.save()

    return True



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import datetime

import ajax_select.fields
import six
from django import forms
from django.conf import settings

from karaage.machines.models import Account, Machine
from karaage.people.utils import (
    UsernameException,
    check_username_for_new_account,
    validate_username_for_new_account,
)


class MachineForm(forms.ModelForm):

    class Meta:
        model = Machine
        fields = (
            'name', 'no_cpus', 'no_nodes', 'type',
            'start_date', 'end_date', 'pbs_server_host',
            'mem_per_core', 'scaling_factor')


class AdminAccountForm(forms.ModelForm):
    username = forms.CharField(
        label=six.u("Requested username"),
        max_length=settings.USERNAME_MAX_LENGTH,
        help_text=((settings.USERNAME_VALIDATION_ERROR_MSG
                    + " and has a max length of %s.")
                   % settings.USERNAME_MAX_LENGTH))
    default_project = ajax_select.fields.AutoCompleteSelectField(
        'project', required=True)
    shell = forms.ChoiceField(choices=settings.SHELLS)

    def __init__(self, person, **kwargs):
        self.person = person
        super(AdminAccountForm, self).__init__(**kwargs)
        self.old_username = self.instance.username

    def clean_username(self):
        username = self.cleaned_data['username']
        try:
            validate_username_for_new_account(self.person, username)
        except UsernameException as e:
            raise forms.ValidationError(e.args[0])
        return username

    def clean_default_project(self):
        data = self.cleaned_data
        if 'default_project' not in data:
            return data
        default_project = data['default_project']

        query = self.person.projects.filter(pk=default_project.pk)
        if query.count() == 0:
            raise forms.ValidationError(
                six.u('Person does not belong to default project.'))

        return default_project

    def clean(self):
        data = self.cleaned_data
        if 'username' not in data:
            return data
        username = data['username']

        if (self.old_username is None
                or self.old_username != username):
            try:
                check_username_for_new_account(self.person, username)
            except UsernameException as e:
                raise forms.ValidationError(e.args[0])

        return data

    def save(self, **kwargs):
        if self.instance.pk is None:
            self.instance.person = self.person
            self.instance.date_created = datetime.date.today()
        return super(AdminAccountForm, self).save(**kwargs)

    class Meta:
        model = Account
        fields = (
            'username',
            'default_project', 'disk_quota', 'shell')


class UserAccountForm(forms.ModelForm):
    shell = forms.ChoiceField(choices=settings.SHELLS)

    class Meta:
        model = Account
        fields = ('shell',)


class AddProjectForm(forms.Form):
    project = ajax_select.fields.AutoCompleteSelectField(
        'project', required=True, label='Add to existing project')



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import datetime

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError

from karaage.plugins.kgsoftware.models import Software, SoftwareVersion

from ...models import CPUJob, UsedModules


class Command(BaseCommand):

    help = ("Link used modules to jobs in the DB. "
            "Defaults to parsing all of yesterdays data")

    def add_arguments(self, parser):
        parser.add_argument(
            '-a', '--all', action='store_true',
            dest='all', default=False,
            help='Report on all months.'),
        parser.add_argument(
            '--start',
            dest='start', default='',
            help='Start date to process modules from'),

    def handle(self, **options):
        today = datetime.date.today()
        yesterday = today - datetime.timedelta(days=1)
        process_all = options.get('all', False)
        start = options.get('start', False)
        verbose = int(options.get('verbosity'))
        ignored_modules = getattr(settings, 'SOFTWARE_IGNORED_MODULES', [])

        if process_all:
            used_modules = UsedModules.objects.all()
        elif start:
            try:
                year, month, day = start.split('-')
                start = datetime.date(int(year), int(month), int(day))
                used_modules = UsedModules.objects.filter(
                    date_added__gte=start)
            except ValueError:
                raise CommandError('Failed to parse start date. '
                                   'Format YYYY-MM-DD')
        else:
            used_modules = UsedModules.objects.filter(date_added=yesterday)

        for um in used_modules:
            try:
                job = CPUJob.objects.get(jobid=um.jobid)
            except CPUJob.DoesNotExist:
                continue
            modules = um.modules.strip().split(':')
            for module in modules:
                if module in ignored_modules:
                    continue
                try:
                    sv = SoftwareVersion.objects.get(module=module)
                except SoftwareVersion.DoesNotExist:
                    try:
                        package_name, version_name = module.split('/')
                    except ValueError:
                        # workaround a bug in Modules
                        try:
                            package_name, version_name = module.split('//')
                        except ValueError:
                            try:
                                print('Failed to create a new software '
                                      'software from module string '
                                      '%s' % module)
                            except UnicodeEncodeError:
                                pass
                            continue
                    software, created = Software.objects.get_or_create(
                        name=package_name)
                    if created and verbose > 0:
                        print("Created new Software software %s" % software)
                    sv, created = SoftwareVersion.objects.get_or_create(
                        software=software, version=version_name)
                    if created and verbose > 0:
                        print("Created new Software version %s" % sv)
                    sv.module = module
                    sv.save()
                if verbose > 1:
                    print("Adding %s to %s" % (sv, job.jobid))
                job.software.add(sv)
                sv.machines.add(job.machine)
                if not sv.last_used or job.date > sv.last_used:
                    sv.last_used = job.date
                    sv.save()

            um.delete()



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible

from karaage.institutes.models import Institute
from karaage.machines.models import Account, Machine
from karaage.people.models import Person
from karaage.plugins.kgsoftware.models import SoftwareVersion
from karaage.projects.models import Project


@python_2_unicode_compatible
class Queue(models.Model):
    name = models.CharField(max_length=50, primary_key=True)
    description = models.CharField(max_length=200, blank=True, null=True)

    class Meta:
        db_table = 'queue'

    def __str__(self):
        return self.name


@python_2_unicode_compatible
class CPUJob(models.Model):
    account = models.ForeignKey(Account, blank=True, null=True, on_delete=models.PROTECT)
    username = models.CharField(max_length=50, blank=True, null=True)
    project = models.ForeignKey(Project, null=True, blank=True, on_delete=models.PROTECT)
    machine = models.ForeignKey(Machine, blank=True, null=True, on_delete=models.PROTECT)
    date = models.DateField(db_index=True, blank=True, null=True)
    queue = models.ForeignKey(Queue, blank=True, null=True, on_delete=models.PROTECT)
    cpu_usage = models.BigIntegerField(blank=True, null=True)
    mem = models.BigIntegerField(blank=True, null=True)
    vmem = models.BigIntegerField(blank=True, null=True)
    ctime = models.DateTimeField(blank=True, null=True)
    qtime = models.DateTimeField(blank=True, null=True)
    etime = models.DateTimeField(blank=True, null=True)
    start = models.DateTimeField(blank=True, null=True)
    act_wall_time = models.BigIntegerField(blank=True, null=True)
    est_wall_time = models.BigIntegerField(blank=True, null=True)
    jobid = models.CharField(max_length=50, blank=True, null=True, unique=True)
    cores = models.BigIntegerField(blank=True, null=True)
    list_mem = models.BigIntegerField(blank=True, null=True)
    list_pmem = models.BigIntegerField(blank=True, null=True)
    list_vmem = models.BigIntegerField(blank=True, null=True)
    list_pvmem = models.BigIntegerField(blank=True, null=True)
    exit_status = models.BigIntegerField(blank=True, null=True)
    jobname = models.CharField(max_length=256, blank=True, null=True)
    software = models.ManyToManyField(SoftwareVersion, blank=True)

    class Meta:
        ordering = ['-date']
        db_table = 'cpu_job'

    def __str__(self):
        if self.jobid:
            return self.jobid
        return ('%s - %s - %s - %s'
                % (self.username, self.project, self.machine, self.date))

    def get_absolute_url(self):
        return reverse('kg_usage_job_detail', args=[self.jobid])

    def wait_time(self):
        diff = self.start - self.qtime
        d = (diff.days * 86400) + diff.seconds
        return d


class UsedModules(models.Model):
    jobid = models.CharField(max_length=100, primary_key=True)
    date_added = models.DateField(auto_now_add=True)
    modules = models.TextField()

    class Meta:
        db_table = 'usage_usedmodules'


class UsageCache(models.Model):
    date = models.DateField(editable=False, auto_now_add=True)
    start = models.DateField()
    end = models.DateField()
    cpu_time = models.DecimalField(max_digits=30, decimal_places=2)
    no_jobs = models.IntegerField()

    class Meta:
        abstract = True


class InstituteCache(UsageCache):
    institute = models.ForeignKey(Institute, on_delete=models.CASCADE)

    class Meta:
        unique_together = (
            'date', 'start', 'end', 'institute')
        db_table = 'cache_institutecache'


class ProjectCache(UsageCache):
    project = models.ForeignKey(Project, on_delete=models.CASCADE)

    class Meta:
        unique_together = (
            'date', 'start', 'end', 'project')
        db_table = 'cache_projectcache'


class PersonCache(UsageCache):
    person = models.ForeignKey(Person, on_delete=models.CASCADE)
    project = models.ForeignKey(Project, on_delete=models.CASCADE)

    class Meta:
        unique_together = (
            'date', 'start', 'end', 'person', 'project')
        db_table = 'cache_personcache'


class MachineCategoryCache(UsageCache):
    available_time = models.DecimalField(max_digits=30, decimal_places=2)

    class Meta:
        unique_together = ('date', 'start', 'end')
        db_table = 'cache_machinecategorycache'


class MachineCache(UsageCache):
    machine = models.ForeignKey(Machine, on_delete=models.CASCADE)

    class Meta:
        unique_together = ('date', 'start', 'end', 'machine')
        db_table = 'cache_machinecache'



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.template.loader import render_to_string

from karaage.common.emails import CONTEXT, send_mail


def render_email(name, context):
    context.update(CONTEXT)
    subject = render_to_string(
        ['karaage/emails/%s_subject.txt' % name,
            'kgapplications/emails/%s_subject.txt' % name],
        context).replace('\n', '')
    body = render_to_string(
        ['karaage/emails/%s_body.txt' % name,
            'kgapplications/emails/%s_body.txt' % name],
        context)
    return subject, body


def _send_request_email(context, role, persons, template):
    # if APPROVE_ACCOUNTS_EMAIL and this email to administrator role, use this
    # value instead of emailing individual administrators.
    if role == "administrator" and \
            settings.APPROVE_ACCOUNTS_EMAIL is not None:

        context['receiver'] = None
        context['receiver_text'] = "Administrator"

        to_email = settings.APPROVE_ACCOUNTS_EMAIL
        subject, body = render_email(template, context)

        send_mail(subject, body, settings.ACCOUNTS_EMAIL, [to_email])
        return

    for person in persons:
        if not person.email:
            continue

        context['receiver'] = person
        context['receiver_text'] = person.get_short_name()

        to_email = person.email
        subject, body = render_email(template, context)

        send_mail(subject, body, settings.ACCOUNTS_EMAIL, [to_email])


def send_request_email(
        authorised_text, authorised_role, authorised_persons, application,
        link, is_secret):
    """Sends an email to admin asking to approve user application"""
    context = CONTEXT.copy()
    context['requester'] = application.applicant
    context['link'] = link
    context['is_secret'] = is_secret
    context['application'] = application
    context['authorised_text'] = authorised_text
    _send_request_email(
        context,
        authorised_role, authorised_persons,
        "common_request")


def send_duplicate_email(
        authorised_text, authorised_role, authorised_persons, application,
        link, is_secret):
    """Sends an email to admin to warn application was marked duplicate."""
    context = CONTEXT.copy()
    context['requester'] = application.applicant
    context['link'] = link
    context['is_secret'] = is_secret
    context['application'] = application
    context['authorised_text'] = authorised_text
    _send_request_email(
        context,
        authorised_role, authorised_persons,
        "project_duplicate_applicant")


def send_invite_email(application, link, is_secret):
    """ Sends an email inviting someone to create an account"""

    if not application.applicant.email:
        return

    context = CONTEXT.copy()
    context['receiver'] = application.applicant
    context['application'] = application
    context['link'] = link
    context['is_secret'] = is_secret

    to_email = application.applicant.email
    subject, body = render_email('common_invite', context)

    send_mail(subject, body, settings.ACCOUNTS_EMAIL, [to_email])


def send_approved_email(
        application, created_person, created_account, link, is_secret):
    """Sends an email informing person application is approved"""
    if not application.applicant.email:
        return

    context = CONTEXT.copy()
    context['receiver'] = application.applicant
    context['application'] = application
    context['created_person'] = created_person
    context['created_account'] = created_account
    context['link'] = link
    context['is_secret'] = is_secret
    subject, body = render_email('common_approved', context)
    to_email = application.applicant.email

    send_mail(subject, body, settings.ACCOUNTS_EMAIL, [to_email])



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.conf.urls import include, url

from karaage.plugins.kgusage import views


urlpatterns = [
    url(r'^$', views.index, name='kg_usage'),

    url(r'^unknown/$', views.unknown_usage, name='kg_usage_unknown'),

    url(r'^search/$', views.search, name='kg_usage_search'),
    url(r'^jobs/$', views.job_list, name='kg_usage_job_list'),
    url(r'^jobs/(?P<jobid>[-.\w\[\]]+)/$',
        views.job_detail, name='kg_usage_job_detail'),

    url(r'^core_report/$', views.core_report, name='kg_usage_core_report'),
    url(r'^mem_report/$', views.mem_report, name='kg_usage_mem_report'),
    url(r'^trends/$', views.institute_trends, name='kg_usage_institute_trends'),

    url(r'^institute/(?P<institute_id>\d+)/$',
        views.institute_usage, name='kg_usage_institute'),
    url(r'^institute/(?P<institute_id>\d+)/users/$',
        views.institute_users, name='kg_usage_users'),
    url(r'^projects/(?P<project_id>%s)/$'
        % settings.PROJECT_VALIDATION_RE,
        views.project_usage, name='kg_usage_project'),

    url(r'^top_users/$', views.top_users, name='kg_usage_top_users'),

    url(r'^software/(?P<software_id>\d+)/stats/$',
        views.software_stats, name='kg_software_stats'),
    url(r'^software_version/(?P<version_id>\d+)/stats/$',
        views.version_stats, name='kg_software_version_stats'),
]

urlpatterns = [
    url(r'^usage/', include(urlpatterns)),
]



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

""" LDAP datastore. """

import logging

import karaage.common.trace as trace
from karaage.datastores import base


logger = logging.getLogger(__name__)


class DataStore(base.DataStore):
    """ LDAP Account and group datastore. """

    def __init__(self, config):
        super(DataStore, self).__init__(config)
        self.value = config.get('value', 'Metrotrains forgot')

    def get_account_details(self, account):
        """ Get the account details. """
        return {
            'type': 'account',
            'answer': '42',
            'value': self.value,
        }

    def account_exists(self, username):
        """ Does the account exist? """
        return False

    def get_group_details(self, group):
        """ Get the group details. """
        return {
            'type': 'group',
            'answer': '42',
            'value': self.value,
        }

    def get_project_details(self, project):
        """ Get project's details. """
        return {
            'type': 'project',
            'answer': '42',
            'value': self.value,
        }

    def get_institute_details(self, institute):
        """ Get institute's details. """
        return {
            'type': 'institute',
            'answer': '42',
            'value': self.value,
        }


trace.attach(trace.trace(logger), DataStore)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.contrib.auth.models import BaseUserManager


class MachineManager(BaseUserManager):

    def authenticate(self, machine_name, password):
        try:
            machine = self.get(name=machine_name)
        except self.model.DoesNotExist:
            return None
        if not machine.check_password(password):
            return None
        return machine


class ActiveMachineManager(MachineManager):

    def get_queryset(self):
        return super(ActiveMachineManager, self) \
            .get_queryset().filter(end_date__isnull=True)



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

import pytest
from django import forms as django_forms
from django.test import TestCase

from karaage.common import forms


@pytest.mark.django_db
class FormsTestCase(TestCase):

    def test_validate_password(self):
        username = "somebody"

        good_password = 'ephiikee0eef2Gae'
        bad_password = 'test'
        old_password = 'JaicaeyaivahD9ph'

        # Long enough.
        res = forms.validate_password(username, good_password)
        self.assertEqual(res, good_password)

        # Passwords match.
        res = forms.validate_password(
            username, good_password, good_password)
        self.assertEqual(res, good_password)

        # New password is different to old password.
        res = forms.validate_password(
            username, good_password, old_password=old_password)
        self.assertEqual(res, good_password)

        # New password is different to old password, passwords match.
        res = forms.validate_password(
            username, good_password, good_password, old_password=old_password)
        self.assertEqual(res, good_password)

        assert_raises = self.assertRaises(django_forms.ValidationError)

        # Too short.
        with assert_raises:
            forms.validate_password(username, bad_password)

        # password contains username
        with assert_raises:
            forms.validate_password(
                username, "2222" + username + "1111")

        # Passwords don't match.
        with assert_raises:
            forms.validate_password(
                username, good_password, good_password + 'diff')

        # New password is the same as old password.
        with assert_raises:
            forms.validate_password(
                username, good_password, old_password=good_password)

        # Second password isn't blank.
        with assert_raises:
            forms.validate_password(
                username, good_password, '')



# Copyright 2010-2017, The University of Melbourne
# Copyright 2010-2017, Brian May
#
# This file is part of Karaage.
#
# Karaage is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Karaage is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Karaage  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.urls import reverse
from django.utils.encoding import python_2_unicode_compatible
from model_utils import FieldTracker

from karaage.common import log
from karaage.machines.models import Machine
from karaage.people.models import Group, Person


@python_2_unicode_compatible
class SoftwareCategory(models.Model):
    name = models.CharField(max_length=100)

    class Meta:
        db_table = 'software_category'
        ordering = ['name']

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('kg_software_category_list')


@python_2_unicode_compatible
class Software(models.Model):
    category = models.ForeignKey(
        SoftwareCategory, blank=True, null=True, on_delete=models.CASCADE)
    name = models.CharField(max_length=200, unique=True)
    description = models.TextField(blank=True, null=True)
    group = models.ForeignKey(
        Group, blank=True, null=True, on_delete=models.SET_NULL)
    homepage = models.URLField(blank=True, null=True)
    tutorial_url = models.URLField(blank=True, null=True)
    academic_only = models.BooleanField(default=False)
    restricted = models.BooleanField(
        help_text="Will require admin approval", default=False)

    _tracker = FieldTracker()

    class Meta:
        ordering = ['name']
        db_table = 'software'

    def save(self, *args, **kwargs):
        created = self.pk is None

        # save the object
        super(Software, self).save(*args, **kwargs)

        if created:
            log.add(self, 'Created')
        for field in self._tracker.changed():
            log.change(self, 'Changed %s to %s'
                       % (field, getattr(self, field)))

    save.alters_data = True

    def delete(self, *args, **kwargs):
        # delete the object
        log.delete(self, 'Deleted')
        super(Software, self).delete(*args, **kwargs)

    delete.alters_data = True

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('kg_software_detail', args=[self.id])

    def get_current_license(self):
        try:
            return self.softwarelicense_set.latest()
        except SoftwareLicense.DoesNotExist:
            return None

    def group_name(self):
        return self.group.name

    def get_group_members(self):
        if self.group is None:
            return Group.objects.none()
        else:
            return self.group.members.all()


@python_2_unicode_compatible
class SoftwareVersion(models.Model):
    software = models.ForeignKey(Software, on_delete=models.CASCADE)
    version = models.CharField(max_length=100)
    machines = models.ManyToManyField(Machine)
    module = models.CharField(max_length=100, blank=True, null=True)
    last_used = models.DateField(blank=True, null=True)

    class Meta:
        db_table = 'software_version'
        ordering = ['-version']

    def __str__(self):
        return '%s - %s' % (self.software.name, self.version)

    def get_absolute_url(self):
        return self.software.get_absolute_url()

    def machine_list(self):
        machines = ''
        if self.machines.all():
            for m in self.machines.all():
                machines += '%s, ' % m.name
        return machines


class SoftwareLicense(models.Model):
    software = models.ForeignKey(Software, on_delete=models.CASCADE)
    version = models.CharField(max_length=100, blank=True, null=True)
    date = models.DateField(blank=True, null=True)
    text = models.TextField()

    class Meta:
        db_table = 'software_license'
        get_latest_by = "date"
        ordering = ['-version']

    def __str__(self):
        return '%s - %s' % (self.software.name, self.version)

    def get_absolute_url(self):
        return reverse('kg_software_license_detail', args=[self.id])


class SoftwareLicenseAgreement(models.Model):
    person = models.ForeignKey(Person, on_delete=models.CASCADE)
    license = models.ForeignKey(SoftwareLicense, on_delete=models.CASCADE)
    date = models.DateField()

    class Meta:
        db_table = 'software_license_agreement'
        get_latest_by = 'date'
